#include "scrapSummaryDisplay.h"

ScrapSummaryDisplay :: ScrapSummaryDisplay(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn, SAMPLE** dataIn, long startSampleIn, long playLengthIn, int numChannelsIn, int summaryNumIn, int numChannelsDisplayedIn): AudioElement(xMinIn, xMaxIn, yMinIn, yMaxIn)
{
	numChannels = numChannelsIn;
	data =	new SAMPLE*[numChannels];
	
	summaryNum = summaryNumIn;
	
	if(playLengthIn > 0 )
	{
		for( int i=0; i < numChannels; i++)
		{
			data[i] = new SAMPLE[playLengthIn];
			
			for( int j=startSampleIn; j < startSampleIn + playLengthIn; j++ )
				data[i][j-startSampleIn] = dataIn[i][j];
		}
		
		dataLength = playLengthIn;
	}
	else	//playLength is < 0, copy in backwards
	{
		for( int i=0; i < numChannels; i++)
		{
			data[i] = new SAMPLE[-playLengthIn];
			
			int copyTo = 0;
			
			for( int j=startSampleIn; j > startSampleIn + playLengthIn; j-- )
				data[i][copyTo++] = dataIn[i][j];
		}
		
		dataLength = -playLengthIn;
	}

	waveformDisplayPaddingY = 4.0;	
	waveformDisplayPaddingXLeft = 35.0;
	waveformDisplayPaddingXRight = 100.0;

	numIndicesPerSummarySample = 1.0;
	usingSummary = false;
	summaryData = new SAMPLE*[numChannels];
	summaryLength = ceil(2 * (xMax - xMin - waveformDisplayPaddingXLeft - waveformDisplayPaddingXRight));
	//For good display performance, we don't want to iterate through more than summaryLength points
	if( dataLength > summaryLength )
	{
		usingSummary = true;
		for( int channel = 0; channel < numChannels; channel++)
		{
			summaryData[channel] = new SAMPLE[summaryLength];
			
			numIndicesPerSummarySample = (1.0 * dataLength) / summaryLength;
			
			SAMPLE currentMin;
			SAMPLE currentMax;
			
			for( int i = 0; i < summaryLength; i++ )
			{
				currentMin = 1.0;
				currentMax = -1.0;
				
				for( int j = floor(i*numIndicesPerSummarySample); j < floor((i+1)*numIndicesPerSummarySample); j++ )
				{
					if( data[channel][j] > currentMax )
						currentMax = data[channel][j];
						
					if( data[channel][j] < currentMin )
						currentMin = data[channel][j];	
				}
	
				summaryData[channel][i++] = currentMin;
				summaryData[channel][i] = currentMax;
			}
		}
	}
	
	//Display all channels if no number was specified
	if( numChannelsDisplayedIn == 0 )
		numChannelsDisplayed = numChannelsIn;
	else
		numChannelsDisplayed = numChannelsDisplayedIn;
	
	//Default to display the entire waveform, start to finish, -1.0 to 1.0 amplitude, with a y = 0 line
	float xMinDisp = xMin + waveformDisplayPaddingXLeft;
	float xMaxDisp = xMax - waveformDisplayPaddingXRight;
	float yMinDisp = yMin + waveformDisplayPaddingY;
	float yMaxDisp = yMax - waveformDisplayPaddingY;
	
	waveformDisplays = new LineDisplay2D*[numChannels];
	if(usingSummary)
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), summaryData[i], summaryLength, 0, summaryLength-1, 1.0, true );
		waveformDisplays[i]->setActive(false);
		}
	}
	else
	{
		for( int i=0; i < numChannels; i++)
		{
			waveformDisplays[i] = new LineDisplay2D( xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)), data[i], dataLength, 0, dataLength-1, 1.0, true );
			waveformDisplays[i]->setActive(false);			
		}
	}


	//Initialize the playhead
	playhead = new Playhead(-1.0, -1.0, yMinDisp, yMaxDisp, 0, dataLength);

	//Initialize the audio player
	audioPlayer = new AudioPlayer(data, dataLength, numChannels, 0, dataLength, false, false);
}



ScrapSummaryDisplay :: ~ScrapSummaryDisplay()
{
	for(int i=0; i < numChannels; i++)
	{
		delete data[i];
		delete summaryData[i];
		delete waveformDisplays[i];
	}
	
	delete []data;
	delete []summaryData;
	delete []waveformDisplays;
	delete playhead;
}


void ScrapSummaryDisplay :: draw()
{
	drawWithOffset( 0.0 );
}


void ScrapSummaryDisplay :: drawWithOffset(float pixelOffsetIn)
{
	float yMinDisp = yMin + pixelOffsetIn;
	float yMaxDisp = yMax + pixelOffsetIn;
	
	if( isActive() )
	  glColor3f( 1.0, 1.0, 1.0);
  else
  	glColor3f( 0.6, 0.6, 0.6);
  	
  glLineWidth( 1.0 );	  	

	//Outer border
 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin, yMinDisp, 0.0);
 		glVertex3f(xMin, yMaxDisp, 0.0);
 		glVertex3f(xMax, yMaxDisp, 0.0);
 		glVertex3f(xMax, yMinDisp, 0.0);
 		glVertex3f(xMin, yMinDisp, 0.0);
	glEnd();

	//Whole box background
	if( isActive() )	
		glColor3f( 0.1, 0.1, 0.1);
	else
		glColor3f( 0.15, 0.15, 0.15);	

	glRectf(xMin+1.0, yMinDisp+1.0, xMax-1.0, yMaxDisp-1.0);
	
	//WaveformDisplay background
	if( isActive() )	
		glColor3f( 0.15, 0.15, 0.15);
	else
		glColor3f( 0.2, 0.2, 0.2);	

	glRectf(xMin+waveformDisplayPaddingXLeft+1.0, yMinDisp+waveformDisplayPaddingY+1.0, xMax-waveformDisplayPaddingXRight-1.0, yMaxDisp-waveformDisplayPaddingY-1.0);



	//Waveform displays border			
	if( isActive() )
	  glColor3f( 0.7, 0.7, 0.7);
  else
  	glColor3f( 0.4, 0.4, 0.4);		
		
 	glBegin( GL_LINE_STRIP );
 		glVertex3f(xMin+waveformDisplayPaddingXLeft, yMinDisp+waveformDisplayPaddingY, 0.0);
 		glVertex3f(xMin+waveformDisplayPaddingXLeft, yMaxDisp-waveformDisplayPaddingY, 0.0);
 		glVertex3f(xMax-waveformDisplayPaddingXRight, yMaxDisp-waveformDisplayPaddingY, 0.0);
 		glVertex3f(xMax-waveformDisplayPaddingXRight, yMinDisp+waveformDisplayPaddingY, 0.0);
 		glVertex3f(xMin+waveformDisplayPaddingXLeft, yMinDisp+waveformDisplayPaddingY, 0.0);
	glEnd();			
	
		
	
	//Waveform displays	
	for( int i = 0; i < numChannelsDisplayed; i++ )
		waveformDisplays[i]->drawWithOffset(pixelOffsetIn);


	float numberBoxXLeft;
	float numberBoxXRight;
	float numberBoxYBottom;
	float numberBoxYTop;
	
	if( summaryNum < 10 )
	{
		numberBoxXLeft = xMin + (waveformDisplayPaddingXLeft/2.0) - 6;	
		numberBoxXRight = xMin + (waveformDisplayPaddingXLeft/2.0) + 6;
		numberBoxYBottom = yMinDisp + (yMaxDisp-yMinDisp)/2.0 - 9.0;
		numberBoxYTop = yMinDisp + (yMaxDisp-yMinDisp)/2.0 + 9.0;
	}
	else
	{
		numberBoxXLeft = xMin + (waveformDisplayPaddingXLeft/2.0) - 10.5;	
		numberBoxXRight = xMin + (waveformDisplayPaddingXLeft/2.0) + 10.5;
		numberBoxYBottom = yMinDisp + (yMaxDisp-yMinDisp)/2.0 - 9.0;
		numberBoxYTop = yMinDisp + (yMaxDisp-yMinDisp)/2.0 + 9.0;		
	}

	//Number box background
	if( isActive() )	
		glColor3f( 0.15, 0.15, 0.15);
	else
		glColor3f( 0.2, 0.2, 0.2);	

	glRectf(numberBoxXLeft + 1.0, numberBoxYBottom + 1.0, numberBoxXRight - 1.0, numberBoxYTop - 1.0);
	
	//Number box border
	if( isActive() )
	  glColor3f( 0.7, 0.7, 0.7);
  else
  	glColor3f( 0.4, 0.4, 0.4);	
  	
 	glBegin( GL_LINE_STRIP );
 		glVertex3f(numberBoxXLeft, numberBoxYBottom, 0.0);
 		glVertex3f(numberBoxXLeft, numberBoxYTop, 0.0);
 		glVertex3f(numberBoxXRight, numberBoxYTop, 0.0);
 		glVertex3f(numberBoxXRight, numberBoxYBottom, 0.0);
 		glVertex3f(numberBoxXLeft, numberBoxYBottom, 0.0);
	glEnd();	  			
	
	//Number
	if( isActive() )
	  glColor3f( 1.0, 1.0, 1.0);
  else
  	glColor3f( 0.6, 0.6, 0.6);	
  	
	char number[10];
	sprintf(number, "%d", summaryNum);
	
	bitmapString( number, numberBoxXLeft + 1.5, numberBoxYBottom + 4.5, 999999, 1 ); 
	
	float lengthMs = (audioPlayer->playLength / (1.0 * SAMPRATE)) * 1000;
	char length[10];
	sprintf(length, "%.2f ms", lengthMs);
	
	bitmapString( length, xMax - waveformDisplayPaddingXRight/2.0 - 30, yMinDisp + 12, 999999 );
		
	if( audioPlayer->isPlaying )
	{
		//Solve for playheadMoving position in terms of display
		float playheadX;
		playheadX = dataIndexValueToXPos(audioPlayer->currentIndex);	
		playhead->setXMin(playheadX);
		
		playhead->drawWithOffset(pixelOffsetIn);
	}
	
	
	if( audioPlayer->isLooping )
	{
		if( isActive() )
			glColor3f( 1.0, 1.0, 1.0 );
		else
			glColor3f( 0.6, 0.6, 0.6 );
			
		bitmapString( "Looping", xMax-waveformDisplayPaddingXRight/2.0 - 22.5, yMaxDisp - 20, 999999 );
	}
}




void ScrapSummaryDisplay :: setActive(bool activeIn)
{
	active = activeIn;
	for( int i=0; i < numChannels; i++ )
		waveformDisplays[i]->setActive(activeIn);
	
	//reset playhead if going inactive so we don't see it
	if( !activeIn )
	{
		playhead->setXMin(-1.0);
		audioPlayer->currentIndex = 0;
		audioPlayer->isPlaying = false;
	}
}


void ScrapSummaryDisplay :: mouseDown(Point2D mouseLoc)
{
	setActive(true);
}


void ScrapSummaryDisplay :: mouseUp(Point2D mouseLoc)
{
	
}


void ScrapSummaryDisplay :: mouseMotion(Point2D mouseLoc)
{
	
}


float ScrapSummaryDisplay :: dataIndexValueToXPos( long indexVal )
{
	float xPosition;
	
	if( usingSummary )
		xPosition = (xMin+waveformDisplayPaddingXLeft) + ( (indexVal / numIndicesPerSummarySample)  / (1.0 * summaryLength) ) * (1.0 * ((xMax-waveformDisplayPaddingXRight) - (xMin+waveformDisplayPaddingXLeft)));
	else
		xPosition = (xMin+waveformDisplayPaddingXLeft) + ( (1.0 * indexVal) / (1.0 * dataLength ) ) * ((xMax-waveformDisplayPaddingXRight) - (xMin+waveformDisplayPaddingXLeft));
	
	return xPosition;
	
}


void ScrapSummaryDisplay :: reverseSelection()
{
	//This shouldn't be called without being active, but just to be careful...
	if( active && !audioPlayer->isPlaying )
	{
		//Actually copy and reverse the audio
		SAMPLE** temp = new SAMPLE*[numChannels];
		
		for( int i=0; i<numChannels; i++)
		{
			temp[i] = new SAMPLE[dataLength];
			
			for( int j=0; j < dataLength; j++)
			{
				temp[i][j] = data[i][dataLength-1-j];
			}
			
			delete data[i];
		}
		
		audioPlayer->bufferPointer = temp;
		delete []data;
		data = temp;
		audioPlayer->bufferPointer = data;
		
		//Re-solve for summary data
		if( usingSummary )
		{
			for( int channel = 0; channel < numChannels; channel++)
			{	
				SAMPLE currentMin;
				SAMPLE currentMax;
				
				for( int i = 0; i < summaryLength; i++ )
				{
					currentMin = 1.0;
					currentMax = -1.0;
					
					for( int j = floor(i*numIndicesPerSummarySample); j < floor((i+1)*numIndicesPerSummarySample); j++ )
					{
						if( data[channel][j] > currentMax )
							currentMax = data[channel][j];
							
						if( data[channel][j] < currentMin )
							currentMin = data[channel][j];	
					}
		
					summaryData[channel][i++] = currentMin;
					summaryData[channel][i] = currentMax;
				}
			}
		}
		
		for( int i=0; i < numChannels; i++ )
			waveformDisplays[i]->generateDisplayVals();
		
	}
	
}


void ScrapSummaryDisplay :: setPosition(float xMinIn, float xMaxIn, float yMinIn, float yMaxIn)
{
	xMin = xMinIn;
	xMax = xMaxIn;
	yMin = yMinIn;
	yMax = yMaxIn;
	
	float xMinDisp = xMin + waveformDisplayPaddingXLeft;
	float xMaxDisp = xMax - waveformDisplayPaddingXRight;
	float yMinDisp = yMin + waveformDisplayPaddingY;
	float yMaxDisp = yMax - waveformDisplayPaddingY;
	
	for( int i=0; i < numChannels; i++ )
	{
		waveformDisplays[i]->resize(xMinDisp, xMaxDisp, yMinDisp+i*(yMaxDisp-yMinDisp)/numChannels, yMinDisp+(1.0-i)*(yMaxDisp-yMinDisp)/numChannels+(i*(yMaxDisp-yMinDisp)));
		waveformDisplays[i]->generateDisplayVals();
	}
		
	playhead->setYPositions(yMinDisp, yMaxDisp);
}